import {
	Function
} from '../static/plugin/common/index.js'

const install = (Vue, vm) => {
	/**
	 * 数据转换
	 */
	const converFunction = (func, data, filter = {}) => {
		if (!func) return
		return new Function('"use strict";return (' + func + ')')()(data, filter);
	}

	/**
	 * 补足数字位数
	 * @param digit 数字
	 * @param len 位数
	 */
	const fillDigit = (digit, len) => {
		return padStart('' + digit, len, '0')
	}

	const toThousands = (value) => {
		let result = ''
		let num = (parseInt(value) || 0).toString()
		while (num.length > 3) {
			result = ',' + num.slice(-3) + result
			num = num.slice(0, num.length - 3)
		}
		if (num) {
			result = num + result
		}
		return result
	}

	const dateFormat = () => {
		let date = new Date();
		let year = date.getFullYear();
		// 在日期格式中，月份是从0开始的，因此要加0，使用三元表达式在小于10的前面加0，以达到格式统一  如 09:11:05
		let month = date.getMonth() + 1 < 10 ? "0" + (date.getMonth() + 1) : date.getMonth() + 1;
		let day = date.getDate() < 10 ? "0" + date.getDate() : date.getDate();
		let hours = date.getHours() < 10 ? "0" + date.getHours() : date.getHours();
		let minutes = date.getMinutes() < 10 ? "0" + date.getMinutes() : date.getMinutes();
		let seconds = date.getSeconds() < 10 ? "0" + date.getSeconds() : date.getSeconds();
		// 拼接
		return year + "-" + month + "-" + day + " " + hours + ":" + minutes + ":" + seconds;
	}

	// px转换为rpx
	const pxToRpx = (value) => {
		return value / uni.getSystemInfoSync().windowWidth * 750
	}

	// rpx转换为px
	const rpxToPx = (value) => {
		return value / 750 * uni.getSystemInfoSync().windowWidth
	}

	const slice = (array, start, end) => {
		let length = array == null ? 0 : array.length
		if (!length) {
			return []
		}
		start = start == null ? 0 : start
		end = end === undefined ? length : end

		if (start < 0) {
			start = -start > length ? 0 : (length + start)
		}
		end = end > length ? length : end
		if (end < 0) {
			end += length
		}
		length = start > end ? 0 : ((end - start) >>> 0)
		start >>>= 0

		let index = -1
		const result = new Array(length)
		while (++index < length) {
			result[index] = array[index + start]
		}
		return result
	}

	/**
	 * 将数组（array）拆分成多个 size 长度的区块，并将这些区块组成一个新数组。
	 * 如果array无法被分割成全部等长的区块，那么最后剩余的元素将组成一个区块。
	 */
	const chunk = (array, size = 1) => {
		const length = array == null ? 0 : array.length
		if (!length || size < 1) {
			return []
		}
		let index = 0
		let resIndex = 0
		const result = new Array(Math.ceil(length / size))

		while (index < length) {
			result[resIndex++] = slice(array, index, (index += size))
		}
		return result
	}
	/**
	 * 从string中提取参数
	 */
	const getQueryFromString = (string) => {
		const num = string.indexOf("?")
		const qs = string.substr(num + 1) // 获取url中"?"符后的字串  
		let args = {} // 保存参数数据的对象
		const items = qs.length ? qs.split("&") : [] // 取得每一个参数项,
		let item = null
		const len = items.length;

		for (var i = 0; i < len; i++) {
			item = items[i].split("=");
			var name = decodeURIComponent(item[0]),
				value = decodeURIComponent(item[1]);
			if (name) {
				args[name] = value;
			}
		}
		return args;
	}

	const filterRequestParams = (params) => {
		let paramStr = JSON.stringify(params)
		if (paramStr.includes('@year')) {
			paramStr = paramStr.replace(/@year/g, yearly())
		}
		if (paramStr.includes('@month')) {
			paramStr = paramStr.replace(/@month/g, monthly())
		}
		if (paramStr.includes('@weekly')) {
			paramStr = paramStr.replace(/@weekly/g, mweekly())
		}
		if (paramStr.includes('@day')) {
			paramStr = paramStr.replace(/@day/g, day())
		}
		if (paramStr.includes('@daily')) {
			const d = daily()
			paramStr = paramStr.replace(/@daily/g, daily())
		}
		if (paramStr.includes('@yesterday')) {
			paramStr = paramStr.replace(/@yesterday/g, yesterday())
		}
		// 匹配以@开头，空格 或者 " 结尾的的字符串
		// const pattern = /((@).*?(\s))|((@).*?("))/g
		// const match = paramStr.matchAll(pattern)
		// if (match?.length) {
		// 	match.forEach((item) => {
		// 		if (item.includes('diffOfNow')) {
		// 			// 取出()里面的内容
		// 			const regex = /(\()\S+(\))/g
		// 			const value = item.matchAll(regex)
		// 			if (value?.length) {
		// 				diffOfNow(parseInt(value[0]))
		// 				const str = '@' + item
		// 				paramStr = paramStr.replace(str, diffOfNow(parseInt(value[0])))
		// 			}
		// 		}
		// 	})
		// }

		// ios会有零宽断言的兼容性问题，废弃掉正则判断，使用强制解析判断
		const reg = '@diffOfNow('
		if (paramStr.includes(reg)) {
			const match = paramStr.split(reg)
			if (match.length >= 2) {
				const str = match[1]
				const nums = str.split(')')
				if (nums.length) {
					const res = reg + nums[0] + ')'
					paramStr = paramStr.replace(res, diffOfNow(parseInt(nums[0])))
				}
			}
		}
		return JSON.parse(paramStr)
	}

	const yearly = () => {
		const date = new Date();
		const year = date.getFullYear();
		return year
	}

	const monthly = () => {
		const date = new Date();
		const month = date.getMonth() + 1 < 10 ? "0" + (date.getMonth() + 1) : date.getMonth() + 1;
		return month
	}

	const weekly = () => {
		let d1 = new Date();
		let d2 = new Date();
		d2.setMonth(0);
		d2.setDate(3); //3 周一为本周第一天；2 周日为本周第一天；1 周六为本周第一天
		let rq = d1 - d2;
		let days = Math.ceil(rq / (24 * 60 * 60 * 1000));
		let weekly = Math.ceil(days / 7);
		return weekly + 1;
	}

	const day = () => {
		const date = new Date();
		const day = date.getDate() < 10 ? "0" + date.getDate() : date.getDate();
		return day
	}

	const daily = () => {
		const date = new Date();
		const year = date.getFullYear();
		// 在日期格式中，月份是从0开始的，因此要加0，使用三元表达式在小于10的前面加0，以达到格式统一  如 09:11:05
		const month = date.getMonth() + 1 < 10 ? "0" + (date.getMonth() + 1) : date.getMonth() + 1;
		const day = date.getDate() < 10 ? "0" + date.getDate() : date.getDate();
		// 拼接
		return year + "-" + month + "-" + day;
	}

	const yesterday = () => {
		return diffOfNow(-1)
	}

	const diffOfNow = (diff) => {
		const date = new Date()
		const timestamp = date.getTime() + diff * 1000 * 24 * 60 * 60
		date.setTime(timestamp)
		const year = date.getFullYear();
		// 在日期格式中，月份是从0开始的，因此要加0，使用三元表达式在小于10的前面加0，以达到格式统一  如 09:11:05
		const month = date.getMonth() + 1 < 10 ? "0" + (date.getMonth() + 1) : date.getMonth() + 1;
		const day = date.getDate() < 10 ? "0" + date.getDate() : date.getDate();
		// 拼接
		return year + "-" + month + "-" + day;

	}

	/**
	 * 
	 * @param tooltip 表单的tooltip设置 
	 */
	const dealTooltip = (tooltip) => {
		if (!tooltip) {
			return
		}
		tooltip = {
			...tooltip,
			position(point, params, dom, rect, size) {
				let x = point[0];
				let y = point[1];
				const {
					contentSize,
					viewSize
				} = size;
				const maxX = viewSize[0] - contentSize[0];
				if (x > maxX) {
					x -= contentSize[0];
				}
				const maxY = viewSize[1] - contentSize[1];
				if (y > maxY) {
					y -= contentSize[1];
				}
				if (y < 0) {
					y = 0
				}
				return [x, y];
			}
		}
		if (tooltip.formatter && tooltip.formatter.length) {
			return {
				...tooltip,
				formatter: (val) => {
					return converFunction(tooltip.formatter, val)
				},
				trigger: "axis"
			}
		}
		return {
			...tooltip,
			trigger: "axis"
		}
	}
	/**
	 * 
	 * @param axis 表单的yAxis设置 
	 */
	const dealYAxis = (axis) => {
		if (!axis) {
			return
		}
		if (axis.showDouble) {
			return [
				dealAxisFormatter({
					...axis
				}),
				dealAxisFormatter({
					...axis,
					...axis.second
				})
			]
		}
		return dealAxisFormatter({
			...axis
		})
	}

	/**
	 * 
	 * @param axis 表单的xAxis设置 
	 */
	const dealXAxis = (axis, dataList) => {
		if (!axis) {
			return
		}
		if (!dataList) {
			return
		}
		const categories = dataList.categories
		if (axis.showDouble) {
			let data = categories
			if (dataList.secondCategories) {
				data = dataList.secondCategories
			}
			return [{
					...dealAxisFormatter({
						...axis
					}),
					data: categories
				},
				{
					...dealAxisFormatter({
						...axis,
						...axis.second
					}),
					data: data
				}
			]
		}
		return {
			...dealAxisFormatter({
				...axis
			}),
			data: categories
		}
	}

	/**
	 * 
	 * @param axis 表单的axis设置 
	 */
	const dealAxisFormatter = (axis) => {
		if (!axis) {
			return
		}
		if (axis.axisLabel.formatter && axis.axisLabel.formatter.length) {
			return {
				...axis,
				axisLabel: {
					...axis.axisLabel,
					formatter: (val) => {
						return converFunction(axis.axisLabel.formatter, val)
					},
				}
			}
		}
		return axis
	}

	const dealAxisIndex = (datas, elementInfo) => {
		const yShowDouble = elementInfo.option.yAxis.showDouble
		let yflag = false
		const xShowDouble = elementInfo.option.xAxis.showDouble
		let xflag = false
		const newData = datas.map((item, index) => {
			let yAxisIndex = 0
			if (yShowDouble) {
				if (item.yAxisIndex) {
					yAxisIndex = item.yAxisIndex
					yflag = true
				}
				// 打开双轴的情况下,没有设置yAxisIndex的，默认数组最后的用右轴
				if (datas.length > 1 && datas.length - 1 === index && !yflag) {
					yAxisIndex = 1
				}
			}
			let xAxisIndex = 0
			if (xShowDouble) {
				if (item.xAxisIndex) {
					xAxisIndex = item.xAxisIndex
					xflag = true
				}
				// 打开双轴的情况下,没有设置xAxisIndex的，默认数组最后的用顶轴
				if (datas.length > 1 && datas.length - 1 === index && !xflag) {
					xAxisIndex = 1
				}
			}
			return {
				...item,
				yAxisIndex: yAxisIndex,
				xAxisIndex: xAxisIndex
			}

		})
		return newData
	}

	/**
	 *  图层关系关联 子组件获取到folderId
	 *  @param {elements}  
	 */
	const elementFolderId = (elements) => {
		elements.forEach(element => {
			if (element.type === 'folder') {
				element.child.data.forEach(item => {
					const index = elements.findIndex(ele => ele.id === item)
					if (index != -1) {
						elements[index].folderId = element.id
						elements[index].isChild = true

					}
				})
			}
		})
		return elements
	}

	vm.$u.common = {
		converFunction,
		fillDigit,
		toThousands,
		dateFormat,
		pxToRpx,
		rpxToPx,
		chunk,
		getQueryFromString,
		filterRequestParams,
		dealTooltip,
		dealXAxis,
		dealYAxis,
		dealAxisIndex,
		elementFolderId
	}
}

export default {
	install
}